<?php
// $Id: daemoncli.php,v 1.2 2010/03/25 19:06:18 pounard Exp $

/**
 * @file
 * Main file for service.
 */

global $stop, $debug;

$debug = FALSE;
$stop = FALSE;

$args = $_SERVER['argv'];

// Parse command line
if (count($args) > 1) {
  foreach ($args as $key => $arg) {
    if ($key) { // Skip first argument, which is command name
      switch ($arg) {
        case "-d":
        case "--debug":
          $debug = TRUE;
          break;

        default:
          print "[ERROR] Wrong command line switch '{$arg}'\n";
          return;
      }
    }
  }
}

// Make sure there is not time limit, in most case, this is useless.
set_time_limit(0);

// Check if launched in cli mode.
if (!isset($_SERVER['REMOTE_ADDR'])) {
  $_SERVER['REMOTE_ADDR'] = '127.0.0.1';
}

// Check for posix and pcntl_* functions.
if (function_exists("pcntl_fork") !== true) {
  print "[ERROR] You must install first php-process package or equivalent";
  return;
}

// Include all defines
require_once dirname(__FILE__) . '/daemoncli.defines.inc';

// Launching process to daemonize service
$pid = pcntl_fork();

switch ($pid) {
  case -1: // Error
    break;
    
  case 0: // Children
    daemonize();
    break;
    
  default: // Parent
    print "[PHP] Launching daemon [pid {$pid}]\n";
    return;
}

/**
 * SIG handler.
 * 
 * @param int $signo
 */
function sig_handler($signo) {
  global $stop;

  switch ($signo) {
    case SIGKILL:
    case SIGTERM:
      require_once './includes/bootstrap.inc';      
      drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
      watchdog("daemoncli", "SIGTERM... Waiting end for process", array(), WATCHDOG_INFO);

      // Changing pid number to check if daemon is running
      variable_set(DAEMONCLI_PID, 0);
      variable_set(DAEMONCLI_LAST_EXECUTE, 0);
      $stop = TRUE;
      break;
  }
}

function daemonize() {
  global $stop;
  $first_time = TRUE;

  // Intercept signals
  declare(ticks = 1);
  pcntl_signal(SIGTERM, "sig_handler");

  while (true) {
    $child_pid = pcntl_fork();

    switch($child_pid) {
      case -1:
        break;

      case 0: // Child process
        require_once './includes/bootstrap.inc';
        drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
        module_load_include('inc', 'daemoncli', 'service_children');
        children_process($first_time);
        exit();
        break;

      default: // Parent process
        $first_time = FALSE;
        pcntl_waitpid($child_pid, $status);
        break;
    }

    if ($stop === TRUE) {
      print "[Daemon] Demande de sortie du programme\n";
      break;
    }

    usleep(DAEMONCLI_DEFAULT_DELAY);
  }

  return;
}
